#!/bin/perl
#################################################################################
# Created       : Martin Foster
# Modified      : 06/05/2001
#################################################################################
#
# Audit - Auditing information for supervisory and administrative views
# Copyright (C) 2000-2001  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################

package Webchat::Audit;						# Package definition

#################################################################################
# Data Members
#################################################################################

#################################################################################
# Sub-Routines
#################################################################################

#####################
# Audit Trail
#
# Used to create an audit trail of all supervisory and administrative changes.  
# This can be useful to see who is overriding a command or granting priviliges 
# without concent or knowledge.

sub AuditTrail							# Database, Parameters
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
        my $cgi      = shift;					# CGI handle	
	my $param    = shift;					# Parameter hash
	my $action   = shift;					# Action to take

	#####################
	# Program area

	# Insert into audit trail
	$Database->do("INSERT INTO Audit 
		(AuditNumber,
		 PuppeteerLogin,
		 PuppeteerHostname,
		 AuditAction,
		 AuditTimestamp)
		VALUES (NULL,?,?,?,NOW())", {}, 
	 $param->{'USER'}, 
	 $cgi->remote_host(),
	 $action);
}

# Alias for bannings
# Add
sub BanAdd
{ AuditTrail(shift, shift, shift, "$_[0]: Addition of puppeteer $_[1] to the banned list..."); }

# Remove
sub BanRem
{ AuditTrail(shift, shift, shift, "$_[0]: Removal of puppeteer $_[1] from the banned list..."); }

# Message Board
sub BoardRem
{ AuditTrail(shift, shift, shift, "$_[0]: Message and all children were purged..."); }

# Documents
sub Document
{ AuditTrail(shift, shift, shift, "$_[0]: Document was altered..."); }

# Alias for Information
sub InfoChange
{ AuditTrail(shift, shift, shift, "$_[0]: Realm information was changed..."); }

# Info reset
sub InfoReset
{ AuditTrail(shift, shift, shift, "$_[0]: Realm information was reset..."); }

# Puppeteer
sub PuppeteerChange
{ AuditTrail(shift, shift, shift, "$_[0]: Puppeteer information was changed..."); }

# Alias for Realm
sub Realm
{ AuditTrail(shift, shift, shift, "$_[0]: The realms primary information was changed/updated..."); }

# Alias for Realm Addition
sub RealmAddition
{ AuditTrail(shift, shift, shift, "$_[0]: The realms created..."); }

# Alias for Realm Removal
sub RealmRemoval
{ AuditTrail(shift, shift, shift, "$_[0]: The realms purged from the system..."); }

# Alais for Tag Changes
sub TagChange
{ AuditTrail(shift, shift, shift, "$_[0]: Tag was altered..."); }

# Alias for Words
sub WordAdd
{ AuditTrail(shift, shift, shift, "$_[0]: Addition of word $_[1] to the word restriction list..."); }

sub WordRem
{ AuditTrail(shift, shift, shift, "$_[0]: Removal of word $_[1] from the word restriction list..."); }

1;
