#################################################################################
# Created       : Martin Foster
# Modified      : 06/11/2001
#################################################################################
#
# Filter - A filtering system that allows for safer handling of posts  
# Copyright (C) 2000-2001  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################

package Webchat::Filter;

#################################################################################
# Sub-Routines
#################################################################################

#####################
# SafeHTML 
#
# While the name may be slightly misleading as the HTML can still be malicious to a degree
# this subroutine will aim to make it safer.   Note however, that the only safe HTML is no
# HTML.

sub SafeHTML 	 						# String to handle 
{
	#####################
	# Data members

	my $str     = shift;					# String name


	#####################
	# Program area
        # Title filters

	# Comments
	$str =~ s/<!-|-->|<COMMENT>|<\/COMMENT>//gsi;

	# Standard filtering 
	$str =~ s/<APPLET.*?>|<\/APPLET>|<BODY.*?>|<\/BODY>|<BGSOUND>|<EMBED.*?>|<\/EMBED>|<FORM.*?>|<\/FORM>|<IMG.*?>|<IMAGE.*?>|<INPUT.*?>//gsi;
	$str =~ s/<HTML.*?>|<\/HTML>|<OPTION.*?>|<SCRIPT.*?>|<TABLE.*?>|<\/TABLE>|<TEXTAREA.*?>|<\/TEXTAREA>|<TITLE.*?>|<\/TITLE>//gsi;

	# Close tags
	while ($str =~ m/<(\w.*?)(\s|>)/gs)
	{
		# Skips
		next if ($1 =~ /^BR/i);
		next if ($1 =~ /^HR/i);
		next if ($1 =~ /^IMG/i);
		next if ($1 =~ /^IMAGE/i);

		# Close tags
		if ($str !~ /<\/$1>/i) { $str = "$str<\/$1>"; }
	}

	# Return
	return $str;
}


#####################
# Strip HTML 
#
# Strip HTML is a harsher addtion to the system.  As this is designed to scrub
# much more of the HTML then the above version.  This is of course to prevent 
# people from adding HTML in things that are useful like a puppet name.

sub StripHTML 	 						# String to handle 
{
	#####################
	# Data members

	my $str     = shift;					# String name


	#####################
	# Program area
        # Title filters

	# Html and special
	$str =~ s/<[^>]*>//gs;
	$str =~ s/\&.*?\;//gs;

	# Comments
	$str =~ s/<!-|-->//gs;

	# Return
	return $str;
}

1;								# Return true
