#!/bin/perl
#################################################################################
# Created       : Martin Foster
# Modified      : 06/03/2001
#################################################################################
#
# Configure Ignore - Script part of Webchat designed to configure ignore lists
# Copyright (C) 2000-2001  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use strict;							# Strict variable enforcement

use Webchat::Database;						# Database handler
use Webchat::Login;						# Login functionality
use Webchat::Param;						# Parameter control
use Webchat::Table;						# Table control


#################################################################################
# Data Members
#################################################################################
my $Database;							# Database handle
my $cgi;							# Common gateway interface handle

my %param;							# Parameter hash
my %config;							# Configuration tag

#################################################################################
# Program Area
#################################################################################

	$cgi = new CGI;						# Create CGI Handle

	Webchat::Database::DatabaseConnect(\$Database);		# Connect to MySQL
	Webchat::Database::GetHashConfiguration($Database, \%config);
	Webchat::Param::GetParam($cgi, \%param);		# Retreive parameters

	print $cgi->header();					# File header
	Webchat::Database::DocumentGetHeader($Database);	# HTML header

	# Authenthication
	if ( Webchat::Login::GetVerificationNormal($Database, $cgi, \%param) )
	{
		print "$config{'TagTitle'}\n<BR>\n";		# Print title
		InitialDisplay($Database, $cgi, \%config, \%param);
	}


	Webchat::Database::DocumentGetFooter($Database);	# HTML footer
	$Database->disconnect();				# Disconnect database handle

#################################################################################
# Sub-Routines
#################################################################################

#####################
# InitialDisplay
#
# Handles the general output of the initial display screen.  Which will allow one
# to change password, E-Mail addresses and/or passwords.

sub InitialDisplay						# Database, CGI, Param
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my $cgi = shift;					# CGI handle

	my $config = shift;					# Configuration hash
	my $param = shift;					# Parameter handle

	my $choice;						# Choice of the matter	
	my $ignored;						# List of the dammed
	my @ignored;						# List of the dammed
	

	#####################
	# Program area	
	
	# Information handling
	# Addition into list
	if (defined($param->{$config->{'TxtSubmitAddition'}}))
	{
		# Determine existence
		($choice) = Webchat::Database::DataGet($Database, 'SELECT PuppetName FROM Puppet WHERE PuppetName=?', $param->{'LIST'});

		# Verification
		if ((length($param->{'LIST'}) > 2) && ($param->{'LIST'} !~ /^\s/) && (defined($choice)))
		{
			# Insert information
			$Database->do("INSERT INTO PuppetIgnore 
				(PuppeteerLogin,
				 PuppetName)
				VALUES (?,?)", {}, 
		 	$param->{'USER'}, $param->{'LIST'});
		}

	}

	# Removal from list
	elsif (defined($param->{$config->{'TxtSubmitDelete'}}))
	{
		# Remove ignored person
		$Database->do("DELETE FROM PuppetIgnore 
			WHERE PuppeteerLogin=? 
			  AND PuppetName=?", {}, 
		 $param->{'USER'}, $param->{'IGNORE'});
	}

	# Cleanup
	delete $param->{$config->{'TxtSubmitAddition'}};
	delete $param->{$config->{'TxtSubmitDelete'}};

	
	# Retreive lists and adjust widgets
	Webchat::Database::GetListPuppetIgnored($Database, \@ignored, $param->{'USER'});
	$ignored = $cgi->scrolling_list('IGNORE', \@ignored, $ignored[0], 10);


	# Ignore list
	# Display of information
	print $cgi->start_form();

	Webchat::Table::MakeTop($cgi);
	Webchat::Table::MakeSingle($cgi, $config->{'TagConfIgnore'});
	Webchat::Table::MakeBlank($cgi);

	# Additions and removal control	
	Webchat::Table::MakeValid($cgi, $cgi->textfield('LIST', '', 20, 30), $cgi->submit($config->{'TxtSubmitAddition'}));
	Webchat::Table::MakeBlank($cgi);
	Webchat::Table::MakeValid($cgi, $ignored, $cgi->submit($config->{'TxtSubmitDelete'}));
	Webchat::Table::MakeBottom($cgi);

	# Embed
	Webchat::Param::EmbedNormal($Database, $cgi, %{$param});
	print $cgi->end_form(), "\n";
}
