#!/bin/perl
#################################################################################
# Created       : Martin Foster
# Modified      : 01/10/2001
#################################################################################
#
# Configure Puppeteer - Script part of Webchat designed to configure Puppeteers
# Copyright (C) 2000-2001  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use Digest::MD5 qw(md5_hex);                                    # MD5 digest cryptographic algorithm
#use strict;							# Strict variable enforcement

use Webchat::Database;						# Database handler
use Webchat::Login;						# Login functionality
use Webchat::Param;						# Parameter control
use Webchat::Table;						# Table control


#################################################################################
# Data Members
#################################################################################
my $Database;							# Database handle
my $cgi;							# Common gateway interface handle

my %param;							# Parameter hash
my %config;							# Configuration tag

#################################################################################
# Program Area
#################################################################################

	$cgi = new CGI;						# Create CGI Handle

	Webchat::Database::DatabaseConnect(\$Database);		# Connect to MySQL
	Webchat::Database::GetHashConfiguration($Database, \%config);
	Webchat::Param::GetParam($cgi, \%param);		# Retreive parameters

	print $cgi->header();					# File header
	Webchat::Database::DocumentGetHeader($Database);	# HTML header

	# Authenthication
	if ( Webchat::Login::GetVerificationNormal($Database, $cgi, \%param) )
	{
		print "$config{'TagTitle'}\n<BR>\n";		# Print title

		# Puppet activity handling
		if (defined($param{'PUPPET'}))
		{ ActivityHandler($Database, $cgi, \%param, \%config); }

		# Initial screen	
		unless (defined($param{'OLDPASS'}) || defined($param{'EMAIL'}))
		{ InitialDisplay($Database, $cgi, \%param, \%config); }

		# Information handling
		else
		{ InformationHandler($Database, $cgi, \%param, \%config); }
	}


	Webchat::Database::DocumentGetFooter($Database);	# HTML footer
	$Database->disconnect();				# Disconnect database handle

#################################################################################
# Sub-Routines
#################################################################################

#####################
# Activity Handler
#
# Will handle the activation de-activation of puppets as designated by the user
# this is to reduce the long list of puppets for selection when some are rarely
# used if ever.

sub ActivityHandler						# Database, CGI, Param
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my $cgi = shift;					# CGI handle
	my $param = shift;					# Parameter handle
	my $config = shift;					# Configuration tags	

	my $puppet;						# Puppet to handle
	my $active;						# Activity flag


	#####################
	# Program area	

	# Find appropraite values
	$active = (defined($param->{$config->{'TxtActive'}})) ? undef : time;
	$puppet = (defined($param->{$config->{'TxtActive'}})) ? $param->{'IPUPPET'}: $param->{'APUPPET'};

	# Update tables
	$Database->do("UPDATE Puppet
		SET PuppetActive=?
		WHERE PuppetName=?
		  AND PuppeteerLogin=?", {},
	 $active,
	 $puppet,
	 $param->{'USER'});

	# Cleanup
	delete $param->{'PUPPET'};
	delete $param->{'APUPPET'};
	delete $param->{'IPUPPET'};
	delete $param->{$config->{'TxtActive'}};
	delete $param->{$config->{'TxtInactive'}};
}


#####################
# Check Data
#  
# Arbitrary sub-routine designed to be used to check validity of data.  Made
# to hide ugliness of checks also centralize adapting of checks.

sub CheckData							# Check value, $choice, Database, User
{
	#####################
	# Data members
	my $value = shift;					# Value to check 
	my $choice = shift;					# Choice of the matter

	my $Database = shift;					# Possible database handle
	my $Statement;						# SQL Statement

	my $user = shift;					# User


	#####################
	# Program area

	# Password Check
	if ($choice == 0)
	{
		# Prepare and execute
		$Statement = $Database->prepare("SELECT PuppeteerPassword 
			FROM  Puppeteer 
			WHERE PuppeteerLogin=?");
		$Statement->execute($user);

		# Retreive and finish
		my $crypt = $Statement->fetchrow();
		$Statement->finish();

		# Verify existence of user and validity of password
		if ( (defined($crypt)) && ($crypt eq crypt($value, $crypt)) )
		{ return 1; }					# Return true
	}

	# Lenght
	elsif ($choice == 1)
	{
		# Lenght of a minumum of 3 characters
		if (length($value) > 3)
		{ return 1; }					# Return true
	}

	# E-Mail
	elsif ($choice == 2)
	{
		# Check for @ and . lenght
		if ( ($value =~ /\@/) && ($value =~ /\./) && (length($value) > 5) )
		{ return 1; }					# Return true
	}

	return 0; 						# False if all else fails
}

# Alias for Password validity
sub CheckDataPassword
{ CheckData($_[2], 0, $_[0], $_[1]); }

# Alias for Lenght
sub CheckDataLenght
{ CheckData(shift, 1); }

# Alias for E-Eail validity
sub CheckDataEMail
{ CheckData(shift, 1); }


#####################
# Information Handler
#
# Will take the parameters and depending on information received can change the
# passwords and email.  The rest is considered non-essential.

sub InformationHandler						# Database, CGI, Param
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my $cgi = shift;					# CGI handle
	my $param = shift;					# Parameter handle
	my $config = shift;					# Configuration hash
	
	my $choice = 0;						# Choice of action


	#####################
	# Program area	
	
	# Display of information
	print $cgi->start_form();
	Webchat::Table::MakeTop($cgi);

	# Primary security check
	# Password Validity
	if ( CheckDataPassword($Database, $param->{'USER'}, $param->{'OLDPASS'}) )
	{ $choice++; }						# Increment by one
	else
	{ Webchat::Table::MakeSingle($cgi, $cgi->h1("<CENTER>$config->{'ErrPassword'}</CENTER>")); }


	# Password change
	if ( defined($param->{'ONEPASS'}) )
	{
		# Valid new passwords
		if ( (CheckDataLenght($param->{'ONEPASS'})) && ($param->{'ONEPASS'} eq $param->{'TWOPASS'}) )
		{ $choice++; }					# Inrement by one
		else
		{ Webchat::Table::MakeSingle($cgi, $cgi->h1("<CENTER>$config->{'ErrMismatch'}</CENTER>")); }
	}	

	# E-Mail change
	elsif ( defined($param->{'EMAIL'}) )
	{
		# Valid new E-Mail
		if ( CheckDataEMail($param->{'EMAIL'}) )
		{ $choice++; }					# Inrement by one
		else
		{ Webchat::Table::MakeSingle($cgi, $cgi->h1("<CENTER>$config->{'ErrEMail'}</CENTER>")); }
	}

	# If both tests pass, change data
	if ($choice == 2)
	{
		# Password change
		if ( defined($param->{'ONEPASS'}) )
		{
			# Create random string
			my $rand = md5_hex(rand());
			$rand = substr($rand, 0, 14);			

			# Insert new password
			$Database->do("UPDATE Puppeteer 
				SET   PuppeteerPassword=? 
				WHERE PuppeteerLogin=?", {}, 
			 crypt($param->{'ONEPASS'}, $rand), $param->{'USER'});
		}
		
		# EMail change
		elsif ( defined($param->{'EMAIL'}) )
		{
			# Insert new password
			$Database->do("UPDATE Puppeteer 
				SET PuppeteerEMail=? 
				WHERE PuppeteerLogin=?", {}, 
			 $param->{'EMAIL'}, $param->{'USER'});
		}

		# Congratulations message
		Webchat::Table::MakeSingle($cgi, $cgi->h1("<CENTER>$config->{'TagConfSuccess'}</CENTER>"));
	}

	# Table bottom
	Webchat::Table::MakeBottom($cgi);
}


#####################
# InitialDisplay
#
# Handles the general output of the initial display screen.  Which will allow one
# to change password, E-Mail addresses and/or passwords.

sub InitialDisplay						# Database, CGI, Param
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my $cgi = shift;					# CGI handle
	my $param = shift;					# Parameter handle
	my $config = shift;					# Configuration hash

	my @active;						# Active handles
	my @inactive;						# Inactive handles
	
	my $active;						# Active handles
	my $inactive;						# Inactive handles

	my %login;						# Login hash
	

	#####################
	# Program area	
	
	# Link with hashes
	Webchat::Database::GetHashLogin($Database, \%login);

	# Populate arrays
	Webchat::Database::GetListPuppetYours($Database, \@active, $param->{'USER'});
	Webchat::Database::GetListPuppetSuspended($Database, \@inactive, $param->{'USER'});

	# Format lists
	$active   = $cgi->scrolling_list('APUPPET', \@active,   $active[0],   5);
	$inactive = $cgi->scrolling_list('IPUPPET', \@inactive, $inactive[0], 5);


	# Puppet activation
	# Display of information
	print $cgi->start_form();

	Webchat::Table::MakeTop($cgi);
	Webchat::Table::MakeSingle($cgi, $config->{'TagConfActivate'});
	Webchat::Table::MakeBlank($cgi);

	# Activation section
	Webchat::Table::MakeSingle($cgi, $config->{'TagActive'});
	Webchat::Table::MakeValid($cgi, $active,   $cgi->submit($config->{'TxtInactive'}));
	Webchat::Table::MakeBlank($cgi);
	Webchat::Table::MakeSingle($cgi, $config->{'TagInactive'});
	Webchat::Table::MakeValid($cgi, $inactive, $cgi->submit($config->{'TxtActive'}));

	# Bottom of page
	Webchat::Table::MakeBottom($cgi);

	# Embed
	Webchat::Param::EmbedNormal($Database, $cgi, %{$param}, 'PUPPET', 'True');
	print $cgi->end_form(), "\n";


	# Password change
	# Spacing
	print "<BR>\n<BR>\n";

	# Display of information
	print $cgi->start_form();

	# Top HTML
	Webchat::Table::MakeTop($cgi);
	Webchat::Table::MakeSingle($cgi, $config->{'TagConfPassword'});
	Webchat::Table::MakeBlank($cgi);

	# Password section
	Webchat::Table::MakeValid($cgi, $login{'TagPasswordOld'}, $cgi->password_field('OLDPASS', '', 10, 8));
	Webchat::Table::MakeValid($cgi, $login{'TagPassword'}, $cgi->password_field('ONEPASS', '', 10, 8));
	Webchat::Table::MakeValid($cgi, $login{'TagPasswordRepeat'}, $cgi->password_field('TWOPASS', '', 10, 8));
	Webchat::Table::MakeBlank($cgi);
	
	# Submit and bottom
	Webchat::Table::MakeValid($cgi, $cgi->submit( $login{'TxtSubmitButtonSub'} ), $cgi->reset( $login{'TxtResetButton'} ));
	Webchat::Table::MakeBottom($cgi);

	# Embed
	Webchat::Param::EmbedNormal($Database, $cgi, %{$param});
	print $cgi->end_form(), "\n";


	# Email address change
	# Spacing
	print "<BR>\n<BR>\n";

	# Display of information
	print $cgi->start_form();

	Webchat::Table::MakeTop($cgi);
	Webchat::Table::MakeSingle($cgi, $config->{'TagConfEMail'});
	Webchat::Table::MakeBlank($cgi);

	# Email section
	Webchat::Table::MakeValid($cgi, $login{'TagPassword'}, $cgi->password_field('OLDPASS', '', 10, 8));
	Webchat::Table::MakeValid($cgi, $login{'TagEMail'}, $cgi->textfield('EMAIL', '', 20, 45));
	Webchat::Table::MakeBlank($cgi);

	# Submit and bottom
	Webchat::Table::MakeValid($cgi, $cgi->submit( $login{'TxtSubmitButtonSub'} ), $cgi->reset( $login{'TxtResetButton'} ));
	Webchat::Table::MakeBottom($cgi);

	# Embed
	Webchat::Param::EmbedNormal($Database, $cgi, %{$param});
	print $cgi->end_form(), "\n";
}
